#include "main.h"
#include "init.h"
#include "io.h"
#ifndef M_PI
#define M_PI 3.141592
#endif

void SDL_init(){
	const SDL_VideoInfo *videoInfo;

	fprintf(stderr,"Initialising SDL video...\n");

	if(SDL_Init(SDL_INIT_VIDEO)<0){
		fprintf(stderr,"Video initialization failed...\n");
		Quit(1);
	}

	videoInfo=SDL_GetVideoInfo();
	if(!videoInfo){
	    	fprintf(stderr,"Video query failed...\n");
	    	Quit(1);
    	}
    	videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
    	videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
    	videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
    	videoFlags |= SDL_RESIZABLE;       /* Enable window resizing */

    	if(videoInfo->hw_available)videoFlags|=SDL_HWSURFACE;
    	else{videoFlags|=SDL_SWSURFACE;}

    	if(videoInfo->blit_hw)videoFlags|=SDL_HWACCEL;
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);

	fprintf(stderr,"Setting SDL video mode...\n");

    	surface=SDL_SetVideoMode(SCREEN_WIDTH,SCREEN_HEIGHT,SCREEN_BPP,videoFlags);
	SDL_WM_SetCaption("GLAsteroids", "v0.9");

    	if(!surface){
		fprintf(stderr,"Video mode set failed...\n");
		Quit(1);
	}
	
	
	if(SDL_InitSubSystem(SDL_INIT_AUDIO)<0){
		fprintf(stderr,"Could not initialize audio subsystem. Turning sound off...\n");
		audio=0;
	}
	else{audio=1;}


     	if(Mix_OpenAudio(22060,AUDIO_S16SYS,2,512)<0){
  		fprintf(stderr,"Unable to open audio. Turning sound off...\n");
		audio=0;
  	}
	else{audio=1;}
}


int resizeWindow(int width, int height){
	GLfloat ratio;
	fprintf(stderr,"Resizing window...\n");
	if(height==0)height=1;
	ratio = (GLfloat)width/(GLfloat)height;
	glViewport(0,0,(GLsizei)width,(GLsizei)height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, ratio, 0.1f, 50000.0f);
    	glMatrixMode(GL_MODELVIEW);
    	glLoadIdentity();
    	return(1);
}

int initGL(){
	fprintf(stderr,"Starting OpenGL...\n");
	glEnable(GL_TEXTURE_2D);
  	glShadeModel(GL_SMOOTH);
  	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  	glClearDepth(1.0f);
  	glEnable(GL_DEPTH_TEST);
  	glEnable(GL_ALPHA_TEST);
  	glAlphaFunc(GL_GEQUAL, 0.5);
  	glDepthFunc(GL_LEQUAL);
  	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	return(1);
}

void create_font(){
	int i,j;
	fontlist=glGenLists(256);
	fprintf(stderr,"Creating font...\n");
	for(j=0;j<6;j++){
	for(i=0;i<16;i++){
		glNewList(31+fontlist+16*j+i,GL_COMPILE);
			glBindTexture(GL_TEXTURE_2D, texture[11]);
			glBegin(GL_QUADS);
				glTexCoord2f((32*i+0)*0.001953,(32*j+1)*0.003906);    
				glVertex2f(0,32);
   		     	glTexCoord2f((32*i+0)*0.001953,(32*j+31)*0.003906);    
				glVertex2f(0,0);
   			   	glTexCoord2f((32*i+32)*0.001953,(32*j+31)*0.003906);    
				glVertex2f(32,0);
       				glTexCoord2f((32*i+32)*0.001953,(32*j+1)*0.003906);    
				glVertex2f(32,32);
			glEnd();
			glTranslated(20, 0, 0);
		glEndList();
	}
	}
}

void create_astd(){

	int i, j, k, numsections=3, sides=4;
	GLfloat step,theta,size=1;
	VECTOR vertices[5][4];

	step=(2*size)/numsections;
	
	for(k=0;k<4;k++){

	for(i=0;i<numsections;i++){
		for(j=0;j<sides;j++){
			theta=2*M_PI*(((GLfloat)(rand())/RAND_MAX)+j)/sides;
			vertices[i][j].x = 1*(i*step-size);
			vertices[i][j].y = 1*sin(theta);	
			vertices[i][j].z = 1*cos(theta);
		}
	}

	glNewList(261+k,GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D, texture[6]);
	
	for(i=0;i<2;i++){
		for(j=0;j<sides-1;j++){
			glBegin(GL_QUADS);

			glTexCoord2f(1.00,0.00);
			glVertex3f(vertices[i][j+1].x,vertices[i][j+1].y,vertices[i][j+1].z);
			glTexCoord2f(0.00,0.00);
			glVertex3f(vertices[i+1][j+1].x,vertices[i+1][j+1].y,vertices[i+1][j+1].z);	
			glTexCoord2f(0.00,1.00);
			glVertex3f(vertices[i+1][j].x,vertices[i+1][j].y,vertices[i+1][j].z);
			glTexCoord2f(1.00,1.00);
			glVertex3f(vertices[i][j].x,vertices[i][j].y,vertices[i][j].z);	
		
  	  		glEnd();
		}
	}

	for(i=0;i<2;i++){		
		glBegin(GL_QUADS);

			glTexCoord2f(1.00,0.00);
			glVertex3f(vertices[i][0].x,vertices[i][0].y,vertices[i][0].z);
			glTexCoord2f(0.00,0.00);
			glVertex3f(vertices[i+1][0].x,vertices[i+1][0].y,vertices[i+1][0].z);	
			glTexCoord2f(0.00,1.00);
			glVertex3f(vertices[i+1][3].x,vertices[i+1][3].y,vertices[i+1][3].z);
			glTexCoord2f(1.00,1.00);
			glVertex3f(vertices[i][3].x,vertices[i][3].y,vertices[i][3].z);	
		
    		glEnd();
	}

	glBegin(GL_QUADS);
		glTexCoord2f(1.00,0.00);
		glVertex3f(vertices[0][0].x,vertices[0][0].y,vertices[0][0].z);
		glTexCoord2f(0.00,0.00);
		glVertex3f(vertices[0][1].x,vertices[0][1].y,vertices[0][1].z);	
		glTexCoord2f(0.00,1.00);
		glVertex3f(vertices[0][2].x,vertices[0][2].y,vertices[0][2].z);
		glTexCoord2f(1.00,1.00);
		glVertex3f(vertices[0][3].x,vertices[0][3].y,vertices[0][3].z);	
	glEnd();

	glBegin(GL_QUADS);
		glTexCoord2f(1.00,0.00);
		glVertex3f(vertices[2][0].x,vertices[2][0].y,vertices[2][0].z);

		glTexCoord2f(0.00,0.00);
		glVertex3f(vertices[2][1].x,vertices[2][1].y,vertices[2][1].z);	
		glTexCoord2f(0.00,1.00);
		glVertex3f(vertices[2][2].x,vertices[2][2].y,vertices[2][2].z);
		glTexCoord2f(1.00,1.00);
		glVertex3f(vertices[2][3].x,vertices[2][3].y,vertices[2][3].z);	
	glEnd();

	glEndList();
	}
}


void create_bull(){	
	glNewList(259,GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D,  texture[7]);
	glBegin(GL_QUADS);
        	glTexCoord2f( 1.00, 0.00 );
		glVertex3f( -1,  1,  0);
		glTexCoord2f( 0.00, 0.00 );
		glVertex3f(  1,  1,  0);
		glTexCoord2f( 0.00, 1.00 );
		glVertex3f(  1, -1,  0);
		glTexCoord2f( 1.00, 1.00 );
		glVertex3f( -1, -1,  0);
	glEnd();
	glEndList();

}

void create_fuel(){	
	GLUquadricObj *pObj = gluNewQuadric();

	glNewList(260,GL_COMPILE);
	glPushMatrix();
    	glBindTexture(GL_TEXTURE_2D, texture[12]);
    		gluQuadricTexture(pObj, 1);
    		gluQuadricDrawStyle(pObj, GLU_FILL);
    	glRotatef(0, 1.0f, 0.0f, 0.0f);
    	glTranslatef(0,0,0 );
    	gluCylinder(pObj, 1, 1, 2, 20, 20);
    	glPopMatrix();
    	gluDeleteQuadric(pObj);
	glEndList();

}

void create_skyb(){
	GLfloat xz=10000.00,y=10000.00;
	GLUquadricObj *pObj = gluNewQuadric();
	
	glNewList(258,GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D, texture[0]);
	glBegin(GL_QUADS);
		glTexCoord2f( 1.00, 0.00 );
  		glVertex3f( -xz, y, xz );
		glTexCoord2f( 1.00, 1.00 );
		glVertex3f(  xz, y, xz );
		glTexCoord2f( 0.00, 1.00 );
		glVertex3f(  xz, y, -xz );
		glTexCoord2f( 0.00, 0.00 );
		glVertex3f( -xz, y, -xz );
    	glEnd();

    	glBindTexture(GL_TEXTURE_2D, texture[1]);
    	glBegin(GL_QUADS);
     		glTexCoord2f( 1.00, 1.00 );
      		glVertex3f( -xz, -y, xz );
        	glTexCoord2f( 1.00, 0.00 );
        	glVertex3f( xz , -y, xz );
        	glTexCoord2f( 0.00, 0.00 );
        	glVertex3f(  xz, -y, -xz );
        	glTexCoord2f( 0.00, 1.00 );
        	glVertex3f( -xz, -y, -xz );

    	glEnd();
    
    	glPushMatrix();
    	glBindTexture(GL_TEXTURE_2D, texture[3]);
    	gluQuadricTexture(pObj, 1);
    	gluQuadricDrawStyle(pObj, GLU_FILL);
    	glRotatef(90, 1.0f, 0.0f, 0.0f);
    	glTranslatef(0,0,-10000 );
    	gluCylinder(pObj, 10000, 10000, 20000, 20, 20);
    	glPopMatrix();
    	gluDeleteQuadric(pObj);
	glEndList();
}

void load_textures(char *file, int i){
	SDL_Surface *text_surface;
	char path[strlen(DATA_DIR)+strlen(file)+2];
	int alpha=GL_RGB;
	sprintf(path,"%s/%s",DATA_DIR,file);
	if((text_surface=IMG_Load(path))){
		glGenTextures(1, &texture[i]);
		glBindTexture(GL_TEXTURE_2D, texture[i]);
		if(text_surface->format->BytesPerPixel==4)alpha=GL_RGBA;
		gluBuild2DMipmaps(GL_TEXTURE_2D,alpha,text_surface->w,
				text_surface->h,alpha,GL_UNSIGNED_BYTE,text_surface->pixels);

		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
         	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
         	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		SDL_FreeSurface(text_surface);
	}
	else{
		fprintf(stderr,"Unable to open %s/%s for loading\n",DATA_DIR,file);
	}
}

void init_objects(){
	int i;

	fprintf(stderr,"Initialising variables...\n");

	/*Sets up inital values of all variables*/

	for(i=0;i<MAX_ASTEROIDS;i++){
		asteroids[i].vx=0;
		asteroids[i].vy=0;
		asteroids[i].vz=0;
		asteroids[i].px=0;
		asteroids[i].py=0;
		asteroids[i].pz=0;
		asteroids[i].xrot=0;
		asteroids[i].yrot=0;
		asteroids[i].zrot=0;
		asteroids[i].vxrot=0;
		asteroids[i].vyrot=0;
		asteroids[i].vzrot=0;
		asteroids[i].state=0;
		asteroids[i].hits=0;
		asteroids[i].prox=0;
	}	

	for(i=0;i!=3;i++){
		srand((unsigned int)(SDL_GetTicks+i+1));
		asteroids[i].vx=ts*(GLfloat)((rand())/RAND_MAX-0.5);             
		asteroids[i].vy=ts*(GLfloat)((rand())/RAND_MAX-0.5);             
		asteroids[i].vz=ts*(GLfloat)((rand())/RAND_MAX-0.5);             
		asteroids[i].px=-pow(-1,i%2)*100*((GLfloat)(rand())/RAND_MAX+3); 
		asteroids[i].py=-pow(-1,i%2)*0*((GLfloat)(rand())/RAND_MAX+3);   
		asteroids[i].pz=-pow(-1,i%2)*100*((GLfloat)(rand())/RAND_MAX+3); 
		asteroids[i].vxrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
		asteroids[i].vyrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
		asteroids[i].vzrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
		asteroids[i].state=3;
		asteroids[i].shape=i; /*dont forget that this could FUCK UP!*/
		asteroids[i].hits=20;
	}
/*
	asteroids[0].vx=10;
	asteroids[0].vy=0;      
	asteroids[0].vz=0;       
	asteroids[0].px=-50;
	asteroids[0].py=0;  
	asteroids[0].pz=-100;
	asteroids[0].vxrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
	asteroids[0].vyrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
	asteroids[0].vzrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
	asteroids[0].state=4;
	asteroids[0].hits=1;

	asteroids[1].vx=-10;           
	asteroids[1].vy=0;            
	asteroids[1].vz=0;          
	asteroids[1].px=50;
	asteroids[1].py=0; 
	asteroids[1].pz=-100;
	asteroids[1].vxrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
	asteroids[1].vyrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
	asteroids[1].vzrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
	asteroids[1].state=1;
	asteroids[1].hits=1;
*/


	for(i=0;i<1;i++){
		ships[i].xrot=0.0;
		ships[i].yrot=0.0;
		ships[i].vx=0.0;
		ships[i].vy=0.0;
		ships[i].vz=0.0;
		ships[i].px=0.0;
		ships[i].py=0.0;
		ships[i].pz=0.0;
		ships[i].vxz=0.0;
		ships[i].fuel=100;
		ships[i].state=1;
	}

	for(i=0;i<128;i++){
		bullets[i].vx=0.0;
		bullets[i].vy=0.0;
		bullets[i].vz=0.0;
		bullets[i].vxz=0.0;
		bullets[i].px=0.0;
		bullets[i].py=0.0;
		bullets[i].pz=0.0;
		bullets[i].state=0;
		bullets[i].range=0;
	}
	
	a=0;s=0;d=0;w=0;space=0;
 	mb1=0,delay=0;
	kills=0,alert=0;
 	timevar=0,reload=6;
	refuel=1,menu=1,finished=1;
}

void load_audio_and_textures(){

	fprintf(stderr,"Loading textures...\n");

	load_textures("sup.jpg",0);
    	load_textures("sdn1.jpg",1);
	load_textures("pz.jpg",2);
    	load_textures("space1.jpg",3);
	load_textures("astlogo.png",4);
    	load_textures("asttex.jpg",6);
    	load_textures("plastex.png",7);    
    	load_textures("aim.png",9);
    	load_textures("font.png",11);
	load_textures("fuel.jpg",12);

	fprintf(stderr,"Creating and compiling models...\n");

    	create_font();
	create_astd();
	create_bull();
	create_skyb();
	create_fuel();

	fprintf(stderr,"Loading audio...\n");

	plasma=NULL;
	proxalert=NULL;
	explo=NULL;
	plasmaChannel=-1;
	proxalertChannel=-1;
	explChannel=-1;

	if(audio){
		if(!(plasma = Mix_LoadWAV("./sounds/plasma.wav")))printf("Could not find plasma.wav\n");
		if(!(proxalert=Mix_LoadWAV("./sounds/ralert.wav")))printf("Could not find ralert.wav\n");
		if(!(explo=Mix_LoadWAV("./sounds/medexpl.wav")))printf("Could not find medexpl.wav\n");
	}
}


int Quit(int ret){
    	if(proxalert)Mix_FreeChunk(proxalert); 
    	if(plasma)Mix_FreeChunk(plasma);
    	if(explo)Mix_FreeChunk(explo);
    	if(audio)Mix_CloseAudio();
    	SDL_Quit();
    	exit(ret);
}
